#!/bin/bash

# $Id: rotatekey.sh 15104 2012-11-08 20:05:17Z yuri $

# USAGE:
#   rotatekey.sh keyname <interval_hrs>

KEYNAME=$1
INTERVAL_HRS=$2 #must be > cronjob_interval + 10minutes
INTERVAL_HRS=${INTERVAL_HRS:-336} #two weeks if not specified
template="${KEYNAME}-XXXXXX"
MAILTO=$LANDER_ADMIN

#just for paths
LANDER_CONFIG=~plander/lander.conf
#read config
if [ -e $LANDER_CONFIG ]; then
    . $LANDER_CONFIG
else
    echo "ERROR: Config file $CONFIG does not exist" >&2
    exit 1
fi

READLINK="readlink -f"

#set umask right away
umask 0077

#sets newkey to the file name created
mknewkey() {
    local tmpkey="newkey"
    newkey=$(mktemp $template) || return 1 #something bad happened
    #  create the key
    rm -f $tmpkey

    $LANDER_SCRUBBER -m -s $tmpkey </dev/null 1>&0 2>&0

    [ -f $tmpkey ] || {
        lander_set_DATE
        echo "$DATE Scrubber failed to create a key"
        exit 1
    }

    mv $tmpkey "$newkey" || return 1
    chmod 600  "$newkey" 
    chown plander:$LANDER_GROUP_RAW $newkey
}

#we rotate every two weeks (cron job runs every hour)
oldlink=$(find $LANDER_ETCDIR/$KEYNAME -mmin +$((INTERVAL_HRS*60-10)) -print 2>/dev/null)
if [ -n "$oldlink" ]; then
    #time to rotate
    oldreal=$($READLINK $oldlink)
    dirname=${oldreal%/*}
    cd $dirname
    #first: generate a new key
    mknewkey || { echo "ERROR: mknewkey failed" >&2; exit 1; }

    #second: make a softlink of the file
    cd $LANDER_ETCDIR
    newlink="$KEYNAME.lnk"
    /bin/ln -fs "$dirname/$newkey" "$newlink"
    
    #third: move the softlink over the old key
    # since the old one was a softlink, it's ok to overwrite it
    mv -f "$newlink" "$oldlink"
    chown -h plander:$LANDER_GROUP_RAW $oldlink 

    lander_set_DATE
    #timestamp the old key for convenience
    echo "$DATE Aged $oldreal, new: $newkey" >>$LANDER_ROTATEKEY_LOGFILE

    #next dagscrubber job will generate a new key
    echo "rotated to newkey: $newkey"
else
    #either too early or files don't exist
    [ -e $LANDER_ETCDIR/$KEYNAME ] && exit 0 #too early
    #don't exist: setup new keys
    case $KEYNAME in
	( key_host ) keydir=$LANDER_HOSTKEYDIR ;;
	( key_full ) keydir=$LANDER_FULLKEYDIR ;;
	( * ) echo "Unexpected keyname for key setup: $KEYNAME" >>$LANDER_ROTATEKEY_LOGFILE; exit 1;
    esac

    [ -d $keydir ] || {
	/usr/bin/install -d --owner=plander --group=$LANDER_GROUP_MAIN --mode=0700 $keydir || exit 1
    }

    cd $keydir

    mknewkey || { echo "ERROR: mknewkey failed" >&2; exit 1; }

    #second: make a softlink of the file
    cd $LANDER_ETCDIR
    keydir=${keydir##*/}

    /bin/ln -fs $keydir/$newkey $KEYNAME
    chown -h plander:$LANDER_GROUP_RAW $KEYNAME 

    lander_set_DATE
    echo "$DATE Created brand new $newkey" >>$LANDER_ROTATEKEY_LOGFILE
fi
