#!/bin/bash

# $Id: scrubber_dispatcher.sh 15918 2013-02-08 01:22:01Z yuri $

LANDER_CONFIG=~plander/lander.conf
#read config
if [ -e $LANDER_CONFIG ]; then
    . $LANDER_CONFIG
else
    echo "ERROR: Config file $LANDER_CONFIG does not exist" >&2
    exit 1
fi

exec >> $LANDER_SCRUBBER_DISPATCHER_LOGFILE 2>&1

landername=$1;

lander_active_users raw
lander_active_users host
lander_active_users flow
lander_active_users full

#first check users, and remove unnecessary files
for datasource in ${landername:-$LANDER_SOURCES}; do
    eval users_rawdata=\$active_raw_${datasource}
    eval users_hostdata=\$active_host_${datasource}
    eval users_flowdata=\$active_flow_${datasource}
    eval users_fulldata=\$active_full_${datasource}

    hook_users=
    for hook in $LANDER_RAW_HOOKS; do
	lander_active_users $hook
	eval users_hook_${hook}=\$active_{$hook}_${datasource}
	eval hook_users+=\$users_hook_${hook}
    done
    if [ -z "$users_flowdata$users_hostdata$users_fulldata$users_rawdata$hook_users" ]; then
	echo "No active users for $datasource"
	lander_queue_wipe_source $datasource
    fi
done

lander_set_DATE
lander_set_JOBS_RUNNING $LOGNAME "$LANDER_SCRUBBER_WORKER_SCRIPT"
CURRENTRUNNING=$JOBS_RUNNING

lander_queue_size $landername; NUMFILES=$_lander_qsize

if [ $NUMFILES -lt $LANDER_MINFILES ] ; then
    [ "$NUMFILES" == "0" ] && exit 0
    head=$(echo "$_lander_cached_queue" | head -1)
    lander_queue_file_older $head $LANDER_MINFILEAGE || exit 0 #ether too young or was picked up
fi

if [ $JOBS_RUNNING -lt $LANDER_MAXSCRUBBERS ]; then
    #we start a new scrubber_worker and let it check the load
    lander_create_job "$LANDER_SCRUBBER_WORKER_SCRIPT" $landername
fi

lander_set_JOBS_RUNNING $LOGNAME "$LANDER_SCRUBBER_WORKER_SCRIPT"

echo "$DATE files:$NUMFILES jobsbefore:$CURRENTRUNNING jobsafter:$JOBS_RUNNING"

exit 0
